<?php

declare(strict_types=1);

namespace apexl\Io\modules\user\RequestHelper;

use apexl\Io\requestHelper\requestHelper;

final readonly class RefreshRequestHelper extends requestHelper
{
    public function validate(): void
    {
        $this->requireBody('refresh_token');
    }

    public function refreshToken(): string
    {
        return $this->bodyParam('refresh_token', '');
    }
}